"""
Library to help managing role based access controls for django apps.
"""

from __future__ import absolute_import, unicode_literals

__version__ = '1.0.0'

default_app_config = 'edx_rbac.apps.EdxRbacConfig'  # pylint: disable=invalid-name
